# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2017-2026, by Samuel Williams.

module Samovar
	# Represents a runtime failure in command execution.
	class Failure < RuntimeError
	end
end
